/* progbase.h -- Library header
   This file is part of PROGBASE.
 
   PROGBASE - A library of core functions to simplify program development
   ----------------------------------------------------------------------
 
   PROGBASE provides a set of core functions to speed up the writing of
   new programs without the need to first build up the program's "base".
   It includes functions for logging, memory management, file operations,
   inter-process communication, linked lists, process handling and time
   based execution.
 
   Copyright (C) 2005 Peter Pramberger
 
   PROGBASE is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License version 2 as
   published by the Free Software Foundation.
 
   PROGBASE is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.
 
   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
   USA.
   ---------------------------------------------------------------------- */

#ifndef PROGBASE_H
# define PROGBASE_H 1

/* Library version */
# ifndef PROGBASE_NAME
#  define PROGBASE_NAME			"progbase"
# endif
# ifndef PROGBASE_VERSION_STR
#  define PROGBASE_VERSION_STR		"0.5.0"
# endif
# ifndef PROGBASE_VERSION_HEX
#  define PROGBASE_VERSION_HEX		0x005200
# endif
# ifndef PROGBASE_VERSION
#  define PROGBASE_VERSION		PROGBASE_VERSION_HEX
# endif
# ifndef PROGBASE_RELEASE_DATE
#  define PROGBASE_RELEASE_DATE		"06-Jul-2005"
# endif

/* GNU C specifics */
# ifndef __GNUC__
#  define __attribute__(x)		/* empty */
# endif

/* Support C++ */
# ifdef BEGIN_C_DECLS
#  undef BEGIN_C_DECLS
#  undef END_C_DECLS
# endif
# ifdef __cplusplus
#  define BEGIN_C_DECLS			extern "C" {
#  define END_C_DECLS			}
# else
#  define BEGIN_C_DECLS			/* empty */
#  define END_C_DECLS			/* empty */
# endif

BEGIN_C_DECLS

/* Required includes */
# include <sys/types.h>
# include <stdio.h>
# include <sys/time.h>
# include <time.h>
# include <stdarg.h>
/* no <stdbool.h> */

/* Required values */
# ifndef NUL
#  define NUL				'\0'
# endif
# ifndef NULL
#  define NULL				(void *)0
# endif
# ifndef VA_DECL
#  define VA_DECL			...
# endif

/* Boolean support */
# if !defined __cplusplus
#  if !defined false
#   if defined __SUNPRO_C && (__SUNPRO_C < 0x550 || __STDC__ == 1)
#    define _Bool			signed char
enum {
  false = 0,
  true = 1
};
#   else
typedef enum {
  _Bool_must_promote_to_int = -1,
  false = 0,
  true = 1
} _Bool;
#    define _Bool			_Bool
#   endif
#  endif
# else
typedef bool				_Bool;
# endif
# define bool				_Bool
# ifndef __bool_true_false_are_defined
#  define false				0
#  define true				1
#  define __bool_true_false_are_defined	1
# endif

/* Required types */
typedef char				pb_char;
typedef unsigned char			pb_uchar;
typedef int				pb_int;
typedef unsigned int			pb_uint;
typedef long				pb_long;
typedef unsigned long			pb_ulong;
typedef bool				pb_bool;
typedef uid_t				pb_uid;
typedef gid_t				pb_gid;
typedef pid_t				pb_pid;
typedef size_t				pb_size;
typedef time_t				pb_time;

/* Global definitions */
# define pb_mem_malloc(num)		_pb_mem_malloc((__FILE__), (__LINE__), (num))
# define pb_mem_calloc(num, size)	_pb_mem_calloc((__FILE__), (__LINE__), (num), (size))
# define pb_mem_realloc(ptr, size)	_pb_mem_realloc((__FILE__), (__LINE__), (ptr), (size))
# define pb_mem_free(ptr)		_pb_mem_free((__FILE__), (__LINE__), (ptr))
# define pb_mem_strdup(string)		_pb_mem_strdup((__FILE__), (__LINE__), (string))
# define pb_mem_lock(ptr)		_pb_mem_lock((__FILE__), (__LINE__), (ptr))
# define pb_mem_unlock(ptr)		_pb_mem_unlock((__FILE__), (__LINE__), (ptr))

# define XMALLOC(type, num)		((type *) pb_mem_malloc ((num) * sizeof (type)))
# define XCALLOC(type, num)		((type *) pb_mem_calloc ((num), sizeof (type)))
# define XREALLOC(type, p, num)		((type *) pb_mem_realloc ((p), (num) * sizeof (type)))
# define XFREE(p)			pb_mem_free (p)
# define XSTRDUP(str)			pb_mem_strdup (str)
# define XSIZEOF(p)			pb_mem_sizeof (p)
# define XLOCK(p)			pb_mem_lock (p)
# define XUNLOCK(p)			pb_mem_unlock (p)
# define XISLOCKED(p)			pb_mem_is_locked (p)
# define XISVALID(p)			pb_mem_is_valid (p)
# define XSCRUB(p)			pb_mem_scrub (p)

/* Initialization */
extern  void	 progbase_init		(const pb_char *, pb_ulong);

/* Module: event */
typedef void	 pb_EVH			(void *);

extern  void	 pb_event_init		(pb_ulong);
extern  void	 pb_event_finish	(void);
extern  pb_bool	 pb_event_get_state	(void);
extern  void	 pb_event_register	(const pb_char *, pb_EVH *, void *, pb_uint, pb_bool);
extern  void	 pb_event_unregister	(pb_EVH *, void *);
extern  pb_bool	 pb_event_find		(pb_EVH *, void *);
extern  void	 pb_event_run		(void);

/* This holds the current time if event subsystem is used */
extern  pb_time	 volatile pb_event_current_time;

/* Module: file */
extern  void	 pb_file_init		(void);
extern  void	 pb_file_finish		(void);
extern  pb_bool	 pb_file_get_state	(void);
extern  FILE	*pb_file_open		(const pb_char *, const pb_char *);
extern  FILE	*pb_file_reopen		(const pb_char *, const pb_char *, FILE *);
extern  void	 pb_file_close		(FILE *);
extern  pb_int	 pb_file_unlink		(const pb_char *);
extern  pb_char *pb_file_find_path	(FILE *);
extern  FILE	*pb_file_find_handle	(const pb_char *);
extern  pb_char	*pb_file_find_mode	(const pb_char *);

extern  pb_bool	 pb_file_exists		(const pb_char *);
extern  pb_bool	 pb_file_is_reg		(const pb_char *);
extern  pb_bool	 pb_file_is_dir		(const pb_char *);
extern  pb_bool	 pb_file_is_chr		(const pb_char *);
extern  pb_bool	 pb_file_is_blk		(const pb_char *);
extern  pb_bool	 pb_file_is_fifo	(const pb_char *);
extern  pb_bool	 pb_file_is_lnk		(const pb_char *);
extern  pb_bool	 pb_file_is_sock	(const pb_char *);

/* Module: ipc */
extern  pb_int	 pb_ipc_mutex_create	(pb_int);
extern  void	 pb_ipc_mutex_remove	(pb_int);
extern  void	 pb_ipc_mutex_lock	(pb_int);
extern  void	 pb_ipc_mutex_unlock	(pb_int);

extern  pb_int	 pb_ipc_sem_create	(pb_int, pb_int);
extern  pb_int	 pb_ipc_sem_create_pub	(key_t, pb_int, pb_int);
extern  pb_int	 pb_ipc_sem_get		(key_t);
extern  void	 pb_ipc_sem_remove	(pb_int);
extern  pb_int	 pb_ipc_sem_getval	(pb_int, pb_int);
extern  pb_int	 pb_ipc_sem_getnval	(pb_int, pb_int);
extern  pb_int	 pb_ipc_sem_getzval	(pb_int, pb_int);
extern  void	 pb_ipc_sem_setval	(pb_int, pb_int, pb_int);
extern  void	 pb_ipc_sem_inc		(pb_int, pb_int, pb_bool);
extern  void	 pb_ipc_sem_incval	(pb_int, pb_int, pb_int, pb_bool);
extern  void	 pb_ipc_sem_dec		(pb_int, pb_int, pb_bool);
extern  void	 pb_ipc_sem_decval	(pb_int, pb_int, pb_int, pb_bool);
extern  void	 pb_ipc_sem_wait	(pb_int, pb_int);

extern  pb_int	 pb_ipc_shm_create	(pb_int, pb_int);
extern  pb_int	 pb_ipc_shm_create_pub	(key_t, pb_int, pb_int);
extern  pb_int	 pb_ipc_shm_get		(key_t);
extern  void	 pb_ipc_shm_remove	(pb_int);
extern  void	*pb_ipc_shm_attach	(pb_int, pb_bool);
extern  void	 pb_ipc_shm_detach	(void *);

extern	pb_int	 pb_ipc_msg_create	(pb_int);
extern	pb_int	 pb_ipc_msg_create_pub	(key_t, pb_int);
extern	pb_int	 pb_ipc_msg_get		(key_t);
extern  void	 pb_ipc_msg_size	(pb_int, pb_int);
extern	void	 pb_ipc_msg_remove	(pb_int);
extern	pb_int	 pb_ipc_msg_send	(pb_int, pb_long, const void *,
	       				 pb_size, pb_int);
extern	pb_int	 pb_ipc_msg_recv	(pb_int, pb_long, void *, pb_size,
	       				 pb_int);

/* Module: list */
typedef struct pb_list
{
  struct pb_list *prev;
  struct pb_list *next;

  pb_uint	  id;
  void		 *userdata;
}
pb_List;

extern  pb_List	*pb_list_new		(void *);
extern  pb_List	*pb_list_add		(pb_List *, void *);
extern  pb_List	*pb_list_insert		(pb_List *, void *);
extern  pb_List	*pb_list_append		(pb_List *, void *);
extern  void	 pb_list_del		(pb_List *);
extern  pb_size	 pb_list_remove		(pb_List *);
extern  pb_List	*pb_list_start		(pb_List *);
extern  pb_List	*pb_list_end		(pb_List *);
extern  pb_List	*pb_list_head		(pb_List *);
extern  pb_List	*pb_list_tail		(pb_List *);
extern  pb_size	 pb_list_length		(pb_List *);
extern  pb_List	*pb_list_nth		(pb_List *, pb_size);
extern  pb_List	*pb_list_find		(pb_List *, void *);

/* Enable this for debugging linked lists */
extern  pb_bool	 pb_list_dump_all;

/* Module: log */
enum
{
  PB_LOG_DBG_EVENT = 0x00400L,		/*  1024 */
  PB_LOG_DBG_FILE  = 0x00800L,		/*  2048 */
  PB_LOG_DBG_IPC   = 0x01000L,		/*  4096 */
  PB_LOG_DBG_LIST  = 0x02000L,		/*  8192 */
  PB_LOG_DBG_MEM   = 0x04000L,		/* 16384 */
  PB_LOG_DBG_PROC  = 0x08000L,		/* 32768 */
  PB_LOG_DBG_USER  = 0x10000L		/* 65536 */
};

extern  void	 pb_log_init		(void);
extern  void	 pb_log_finish		(void);
extern  FILE	*pb_log_open_file	(const pb_char *, pb_bool, pb_bool);
extern  FILE	*pb_log_reopen_file	(FILE *);
extern  void	 pb_log_close_file	(FILE *);
extern  pb_bool	 pb_log_open_syslog	(pb_int, pb_int);
extern  void	 pb_log_close_syslog	(void);
extern  pb_bool	 pb_log_get_state	(void);
extern  void	 pb_log_set_debug	(pb_ulong);
extern  pb_char	*pb_log_set_name	(const pb_char *);

extern  void	 pb_log_debug		(FILE *, pb_ulong, const pb_char *, VA_DECL)
					 __attribute__ ((format (printf, 3, 4)));
extern  void	 pb_log_info		(FILE *, const pb_char *, VA_DECL)
					 __attribute__ ((format (printf, 2, 3)));
extern  void	 pb_log_warning		(FILE *, const pb_char *, VA_DECL)
					 __attribute__ ((format (printf, 2, 3)));
extern  void	 pb_log_error		(FILE *, const pb_char *, VA_DECL)
					 __attribute__ ((format (printf, 2, 3)));
extern  void	 pb_log_fatal		(FILE *, pb_int, const pb_char *, VA_DECL)
					 __attribute__ ((format (printf, 3, 4), noreturn));

/* This holds the process name for logging */
extern  pb_char	*pb_log_procname;

/* Module: mem */
extern  void	 pb_mem_init		(void);
extern  void	 pb_mem_finish		(void);
extern  pb_bool	 pb_mem_get_state	(void);
extern  void	*_pb_mem_malloc		(const pb_char *, pb_uint, const pb_size);
extern  void	*_pb_mem_calloc		(const pb_char *, pb_uint, const pb_size, const pb_size);
extern  void	*_pb_mem_realloc	(const pb_char *, pb_uint, const void *, const pb_size);
extern  void	 _pb_mem_free		(const pb_char *, pb_uint, void *);
extern  pb_char	*_pb_mem_strdup		(const pb_char *, pb_uint, const pb_char *);
extern  void	 _pb_mem_lock		(const pb_char *, pb_uint, const void *);
extern  void	 _pb_mem_unlock		(const pb_char *, pb_uint, const void *);

extern  pb_bool	 pb_mem_is_locked	(const void *);
extern  pb_bool	 pb_mem_is_valid	(const void *);
extern  pb_size	 pb_mem_sizeof		(const void *);
extern	void	 pb_mem_scrub		(const void *);

/* Call this function on memory exhaustion */
extern  void   (*pb_mem_alloc_fail)	(void);

/* Module: proc */
extern  void	 pb_proc_temp_uid	(pb_uid, pb_gid);
extern  void	 pb_proc_rest_uid	(void);
extern  void	 pb_proc_perm_uid	(pb_uid, pb_gid);

extern  void	 pb_proc_daemonize	(void);
extern  void	 pb_proc_write_pid	(const pb_char *);
extern  void	 pb_proc_sleep		(pb_time);
extern	void	 pb_proc_init_proctitle	(pb_int, pb_char *[], pb_char *[]);
extern	void	 pb_proc_set_proctitle	(const pb_char *, VA_DECL)
					 __attribute__ ((format (printf, 1, 2)));

/* Module: user */
extern  pb_uid	 pb_user_get_uid	(const pb_char *);
extern  pb_gid	 pb_user_get_gid	(const pb_char *);
extern  pb_char	*pb_user_get_name	(pb_uid);
extern  pb_char	*pb_user_get_home	(pb_uid);
extern  pb_char	*pb_user_get_shell	(pb_uid);

extern  pb_gid	 pb_group_get_gid	(const pb_char *);
extern  pb_char	*pb_group_get_name	(pb_gid);

END_C_DECLS

#endif /* not PROGBASE_H */
